"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertSvelteRoot = void 0;
const compiler_1 = require("svelte/compiler");
const sort_1 = require("../sort");
const element_1 = require("./element");
const attr_1 = require("./attr");
/**
 * Convert root
 */
function convertSvelteRoot(svelteAst, ctx) {
    var _a, _b;
    const ast = Object.assign({ type: "Program", body: [], comments: ctx.comments, sourceType: "module", tokens: ctx.tokens, parent: null }, ctx.getConvertLocation({ start: 0, end: ctx.code.length }));
    const body = ast.body;
    if (svelteAst.html) {
        const fragment = svelteAst.html;
        body.push(...element_1.convertChildren(fragment, ast, ctx));
    }
    if (svelteAst.instance) {
        const instance = svelteAst.instance;
        const script = Object.assign({ type: "SvelteScriptElement", name: null, attributes: [], body: [], parent: ast }, ctx.getConvertLocation(instance));
        extractAttributes(script, ctx);
        element_1.extractElementTokens(script, ctx, {
            buildNameNode: (openTokenRange) => {
                ctx.addToken("HTMLIdentifier", openTokenRange);
                const name = Object.assign({ type: "SvelteName", name: "script", parent: script }, ctx.getConvertLocation(openTokenRange));
                return name;
            },
        });
        body.push(script);
    }
    if (svelteAst.module) {
        const module = svelteAst.module;
        const script = Object.assign({ type: "SvelteScriptElement", name: null, attributes: [], body: [], parent: ast }, ctx.getConvertLocation(module));
        extractAttributes(script, ctx);
        element_1.extractElementTokens(script, ctx, {
            buildNameNode: (openTokenRange) => {
                ctx.addToken("HTMLIdentifier", openTokenRange);
                const name = Object.assign({ type: "SvelteName", name: "script", parent: script }, ctx.getConvertLocation(openTokenRange));
                return name;
            },
        });
        body.push(script);
    }
    if (svelteAst.css) {
        const style = Object.assign({ type: "SvelteStyleElement", name: null, attributes: [], children: [], parent: ast }, ctx.getConvertLocation(svelteAst.css));
        extractAttributes(style, ctx);
        element_1.extractElementTokens(style, ctx, {
            buildNameNode: (openTokenRange) => {
                ctx.addToken("HTMLIdentifier", openTokenRange);
                const name = Object.assign({ type: "SvelteName", name: "script", parent: style }, ctx.getConvertLocation(openTokenRange));
                return name;
            },
        });
        const contentsStart = ctx.code.indexOf(">", (_b = (_a = style.attributes[style.attributes.length - 1]) === null || _a === void 0 ? void 0 : _a.range[1]) !== null && _b !== void 0 ? _b : style.name.range[1]) + 1;
        const contentsEnd = ctx.code.lastIndexOf("</style", style.range[1]);
        ctx.addToken("HTMLText", { start: contentsStart, end: contentsEnd });
        body.push(style);
    }
    const useRanges = sort_1.sort([...ctx.tokens, ...ctx.comments]).map((t) => t.range);
    let range = useRanges.shift();
    for (let index = 0; index < ctx.sourceCode.svelte.length; index++) {
        while (range && range[1] <= index) {
            range = useRanges.shift();
        }
        if (range && range[0] <= index) {
            index = range[1] - 1;
            continue;
        }
        const c = ctx.sourceCode.svelte[index];
        if (!c.trim()) {
            continue;
        }
        if (isPunctuator(c)) {
            ctx.addToken("Punctuator", { start: index, end: index + 1 });
        }
        else {
            // unknown
            // It is may be a bug.
            ctx.addToken("Identifier", { start: index, end: index + 1 });
        }
    }
    sort_1.sort(ctx.comments);
    sort_1.sort(ctx.tokens);
    return ast;
    /**
     * Checks if the given char is punctuator
     */
    function isPunctuator(c) {
        return /^[^\w$]$/iu.test(c);
    }
}
exports.convertSvelteRoot = convertSvelteRoot;
/** Extract attrs */
function extractAttributes(element, ctx) {
    const script = element.type === "SvelteScriptElement";
    const code = ctx.sourceCode.svelte.slice(0, element.range[0]).replace(/./g, " ") +
        ctx.sourceCode.svelte
            .slice(...element.range)
            .replace(script
            ? /<script(\s[\s\S]*?)?>([\s\S]*?)<\/script>/giu
            : /<style(\s[\s\S]*?)?>([\s\S]*?)<\/style>/giu, (_tag, attributes, context) => `${script ? "<div   " : "<div  "}${attributes || ""}>${" ".repeat(context.length)}</div>`);
    const svelteAst = compiler_1.parse(code);
    const fakeElement = svelteAst.html.children.find((c) => c.type === "Element");
    element.attributes.push(...attr_1.convertAttributes(fakeElement.attributes, element, ctx));
}
